REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off

rem getting input parameters
setlocal EnableDelayedExpansion

set TEMP_FILE=%1
set DOMAIN=%~4
set LDAP_CONFIG=%~3
set InstanceName=%5
set PROPS_FILE=%6
set Path=%TSIEM_HOME%\jre\jre\bin;%Path%

set ACTUATOR_LOG="..\log\%PROPS_FILE%.log"
set LINE_NUMBER=-1


rmdir %PROPS_FILE%.dir /S /q
mkdir %PROPS_FILE%.dir

rem checking parameters
if %TEMP_FILE%=="" (
 echo %date% %time% Output file is not defined >> %ACTUATOR_LOG%
 exit /b 1
)
type nul > %TEMP_FILE%

if %InstanceName%=="" (
 echo %date% %time% Instance name is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

if %PROPS_FILE%=="" (
 echo %date% %time% PROPS_FILE is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

rem for different DB2 copies
set db2_tool=db2
set db2cmd_tool=db2cmd
set /A InstanceFound=0
set ILIST="%PROPS_FILE%.dir\list.txt"
set JAVA_BIN=java
rem end

rem setting LDAP parameters
set DBM_CFG="%PROPS_FILE%.dir\db2_cfg.txt"
set /A LDAP_ENABLED=0
set LDAP_HOST=127.0.0.1
set LDAP_PORT=389
set USER_OBJECTCLASS=inetOrgPerson
set GROUP_OBJECTCLASS=groupOfNames
set USER_BASEDN=c=*
set GROUP_BASEDN=c=*
set USERID_ATTRIBUTE=uid
set AUTHID_ATTRIBUTE=uid
set GROUPNAME_ATTRIBUTE=cn
set SEARCH_DN=
set SEARCH_PW=none
set GROUP_LOOKUP_METHOD=SEARCH_BY_DN
set GROUP_LOOKUP_ATTRIBUTE=member

rem setting role's variables
set ROLE_LIST_TMP="%PROPS_FILE%.dir\role_list_tmp.txt"
set REQUEST_FILE="%PROPS_FILE%.dir\request.txt"
set ROLE_LIST="%PROPS_FILE%.dir\roles.txt"
set USER_LIST="%PROPS_FILE%.dir\users.txt"
set ROLES_TABLE_START="%PROPS_FILE%.dir\roles_table_start.txt"
set ROLES_TABLE_CURRENT="%PROPS_FILE%.dir\roles_table_current.txt"
set ROLES_TABLE_END="%PROPS_FILE%.dir\roles_table_end.txt"
set REPLACES="%PROPS_FILE%.dir\replaces.txt"
set TEMPFILE="%PROPS_FILE%.dir\tempfile.txt"
set TEMPFILE2="%PROPS_FILE%.dir\tempfile2.txt"
set OUTPUT_FILE="%PROPS_FILE%.dir\roles_grouping.txt"
set LDAP_OUTPUT_FILE="%PROPS_FILE%.dir\ldap_output_file.txt"


rem switching to the target instance
set DB2INSTANCE=%InstanceName%
db2cmd.exe -c -w "db2 -r %TEMPFILE% get instance" 2>> %ACTUATOR_LOG%
if errorlevel 1 (
   db2swtch -l > %ILIST% 2>> %ACTUATOR_LOG%
   for /F "tokens=1,*" %%i in ('find /v /i "default" %ILIST%') do (
       if %%j GTR . (
          if not "%%j" == "%ILIST%" (
             for /F "delims=" %%k in ('echo %%j\') do (
                 for /F "delims=" %%x in ('"%%~pdkbin\db2ilist"') do (
                     if "%%x" == "%InstanceName%" (
                        set db2cmd_tool=%%~pdkbin\db2cmd.exe
                        set InstanceFound=1
                     )
                 )
             )
          )
       )
   )
) else (
   for /F "tokens=1,2 delims=:" %%i in ('type %TEMPFILE%') do (
       if not "  %InstanceName%"=="%%j" (
          db2swtch -l > %ILIST% 2>> %ACTUATOR_LOG%
          for /F "tokens=1,*" %%i in ('find /v /i "default" %ILIST%') do (
              if %%j GTR . (
                 if not "%%j" == "%ILIST%" (
                    for /F  "delims=" %%k in ('echo %%j\') do (
                        for /F "delims=" %%x in ('"%%~pdkbin\db2ilist"') do (
                            if "%%x" == "%InstanceName%" (
                               set db2cmd_tool=%%~pdkbin\db2cmd.exe
                               set InstanceFound=1
                            )
                        )
                    )
                 )
              )
          )
       ) else (
          set /A InstanceFound=1
      )
   )
)

if %InstanceFound% EQU 0 (
   echo %date% %time% Instance %InstanceName% does not exists.>> %ACTUATOR_LOG%
   echo %date% %time% Unable to attach to the instance. >> %ACTUATOR_LOG%
   echo %date% %time% Default instance will be used. >> %ACTUATOR_LOG%
) else (
    echo %date% %time% Checking existing of db2cmd utility for instance %InstanceName%>> %ACTUATOR_LOG%
   "%db2cmd_tool%" -c -w "db2 -r %TEMPFILE% get instance" 2>> %ACTUATOR_LOG%
   if errorlevel 1 (
      echo %date% %time% Instance %InstanceName% does not exists.>> %ACTUATOR_LOG%
      echo %date% %time% Unable to attach to the instance. >> %ACTUATOR_LOG%
      echo %date% %time% Default instance will be used. >> %ACTUATOR_LOG%
   )
)   
 
set DATABASE_LIST_TMP="%PROPS_FILE%.dir\database_list_tmp.txt"
rem the same - final list
set DATABASE_LIST="%PROPS_FILE%.dir\database_list.txt"
set /A GET_ROLES=0
set /A GET_USERS=0

echo %date% %time% Processing started. > %ACTUATOR_LOG%
rem checking if LDAP is enabled
if exist %DBM_CFG% del %DBM_CFG%
"%db2cmd_tool%" -c -w "db2 -r %DBM_CFG% get dbm cfg" 2>> %ACTUATOR_LOG%
if errorlevel 1 (
   echo %date% %time% Unable to call db2 and get DBM configuration.>> %ACTUATOR_LOG%
) else (
   set MARK=0
   for /F "delims== tokens=1,2" %%i in ('type %DBM_CFG%') do (
       set VALUE=%%j
       if !LDAP_ENABLED! EQU 0 (
          for /f %%k in ('echo %%i ^| find /c "(CLNT_PW_PLUGIN)"') do set MARK=%%k
          if !MARK! EQU 1 (
             if "!VALUE!" == " IBMLDAPauthclient" (
                set /A LDAP_ENABLED=1
                goto out1
             )
          )
       )   
       if !LDAP_ENABLED! EQU 0 (
          for /f %%k in ('echo %%i ^| find /c "(SRVCON_PW_PLUGIN)"') do set MARK=%%k
          if !MARK! EQU 1 (
             if "!VALUE!" == " IBMLDAPauthclient" (
                set /A LDAP_ENABLED=1
                goto out1
             )
          )
       )   
   )
)
:out1
    
if %LDAP_ENABLED% EQU 1 (    
   if "%LDAP_CONFIG%"=="" (
      echo %date% %time% LDAP config path is not defined. >> %ACTUATOR_LOG%
   )
) else (
   if "%DOMAIN%"=="" (
      echo %date% %time% Domain is not defined. >> %ACTUATOR_LOG%
   )
)
rem parsing LDAP_CONFIG for actual values

if %LDAP_ENABLED% EQU 1 (
     if exist "%LDAP_CONFIG%" (
        echo %date% %time% Collecting LDAP users and groups. >> %ACTUATOR_LOG%
        for /F "eol=; delims== tokens=1,*" %%i in ('type "%LDAP_CONFIG%"') do (
            set MARK=0
            for %%x in (%%i) do set VAR_NAME=%%x
            for /f "tokens=*" %%x in ("%%j") do set VAR_VALUE=%%~x
            if not "!VAR_NAME!" == "" call :get_ldap_conf
        )
    )
)

for /F "delims=: tokens=1,2" %%x in ("%LDAP_HOST%") do (
    set LDAP_HOST=%%x
    set LDAP_PORT=%%y
)

if %LDAP_ENABLED% EQU 1 (
   if exist "%LDAP_CONFIG%" (
      java -classpath "..\bin\getViper2ldapgroups.jar" com.ibm.tcim.cea.gensub.db2viper.java.CollectViperGroups -LDAPHost?"!LDAP_HOST!" -LDAPPort?"!LDAP_PORT!" -UserName?"!SEARCH_DN!" -Password?"!SEARCH_PW!" -USER_BASEDN?"!USER_BASEDN!" -AUTHID_ATTRIBUTE?"!AUTHID_ATTRIBUTE!" -USER_OBJECTCLASS?"!USER_OBJECTCLASS!" -GROUP_OBJECTCLASS?"!GROUP_OBJECTCLASS!" -GROUP_BASEDN?"!GROUP_BASEDN!" -GROUPNAME_ATTRIBUTE?"!GROUPNAME_ATTRIBUTE!" -GROUP_LOOKUP_METHOD?"!GROUP_LOOKUP_METHOD!" -GROUP_LOOKUP_ATTRIBUTE?"!GROUP_LOOKUP_ATTRIBUTE!" -Filename?"%LDAP_OUTPUT_FILE%" 2>> %ACTUATOR_LOG%
      if errorlevel 1 (
         echo %date% %time% Unable to collect LDAP users.>> %ACTUATOR_LOG%
      ) else ( echo %date% %time% LDAP information is collected successfully.>> %ACTUATOR_LOG% )
   ) else (
      echo %date% %time% LDAP config file does not exist. >> %ACTUATOR_LOG%
   )
) else (
   if not "%DOMAIN%"=="" (
      echo %date% %time% Collecting OS and domain users. >> %ACTUATOR_LOG%
      call ..\bin\getgroupsAD.bat %TEMP_FILE% "" "" "" "" %DOMAIN% "" "" "" "" "" "" %PROPS_FILE% 2>> %ACTUATOR_LOG%
      if errorlevel 1 (
         echo %date% %time% Unable to collect domain and OS groups and users.>> %ACTUATOR_LOG%
      ) else (
         echo %date% %time% Domain and OS information is collected successfully.>> %ACTUATOR_LOG%
      )
   )  
)

rem collecting roles
echo ^[SourceGroup^]> %OUTPUT_FILE% 
echo %date% %time% Start collecting roles.>> %ACTUATOR_LOG%

rem getting list of the databases
"%db2cmd_tool%" -c -w "db2 -r %DATABASE_LIST_TMP% list database directory" 2>> %ACTUATOR_LOG%
if errorlevel 1 (
   echo %date% %time% Unable to call db2. Exiting.>> %ACTUATOR_LOG%
   exit /b 1
)
 
for /f "delims=" %%I in ('type %DATABASE_LIST_TMP%') do if not "%%I" == "" call :get_dbnames "%%I"

rem getting roles for each database

for /F %%c in ('type %DATABASE_LIST%') do (
    set DBNAME=%%c
    echo %date% %time% Working for database !DBNAME!.>> %ACTUATOR_LOG%

    rem getting users
    type nul > %USER_LIST%
    "%db2cmd_tool%" -c -w "db2 connect to !DBNAME! && db2 -xt  -r %USER_LIST% select CONCAT(CONCAT(TRIM(CHAR(grantee)), ','), CONCAT(TRIM(CHAR(rolename)), ',')) from sysibm.sysroleauth where granteetype = 'U'" 2>> %ACTUATOR_LOG%
    if errorlevel 2 (
       echo %date% %time% Unable to call db2.>> %ACTUATOR_LOG%
    ) else (
       if errorlevel 1 (
          echo %date% %time% There is no data in sysibm.sysroleauth table.>> %ACTUATOR_LOG%
       ) else (
          echo %date% %time% Roles are collected successfully.>> %ACTUATOR_LOG%
       )
    )
     
    rem preparing grouping file
    for /F "delims=, tokens=1,2" %%i in ('type %USER_LIST%') do (
    
      
        
        echo "!DBNAME!\%%j">> %OUTPUT_FILE%
        echo {>> %OUTPUT_FILE%
        echo    match logonname "%%i">> %OUTPUT_FILE%
        echo    match instancename "%InstanceName%">> %OUTPUT_FILE%
        echo }>> %OUTPUT_FILE%
         
      )
)

rem grouping file is ready here

echo %date% %time% Finished collecting roles.>> %ACTUATOR_LOG%

if exist %OUTPUT_FILE% type %OUTPUT_FILE% >> %TEMP_FILE%
if exist %LDAP_OUTPUT_FILE% type %LDAP_OUTPUT_FILE% >> %TEMP_FILE%

echo %date% %time% Collect of user and group information is successful. >> %ACTUATOR_LOG%

rmdir %PROPS_FILE%.dir /S /q
echo %date% %time% Processing finished. >> %ACTUATOR_LOG%
exit /b 0

:get_dbnames
set STR=%1
for /f %%I in ('echo %STR% ^| find /c "="') do set MARK=%%I
if %MARK% EQU 0 exit /b
set /a LINE_NUMBER+=1
if %LINE_NUMBER% LSS 1 exit /b
if %LINE_NUMBER% EQU 9 (
  set LINE_NUMBER=0
  exit /b
)
if %LINE_NUMBER% GTR 1 exit /b
for /f "tokens=2 delims==" %%I in ('echo %STR%') do set DBNAME=%%I
set DBNAME=%DBNAME:~1,-1%
echo %DBNAME%>>%DATABASE_LIST%
exit /b


:get_ldap_conf
if "%VAR_NAME%" == "LDAP_HOST" goto trim_spaces
if "%VAR_NAME%" == "USER_OBJECTCLASS" goto trim_spaces
if "%VAR_NAME%" == "GROUP_OBJECTCLASS" goto trim_spaces
if "%VAR_NAME%" == "AUTHID_ATTRIBUTE" goto trim_spaces
if "%VAR_NAME%" == "GROUPNAME_ATTRIBUTE" goto trim_spaces
if "%VAR_NAME%" == "GROUP_LOOKUP_METHOD" goto trim_spaces
if "%VAR_NAME%" == "GROUP_LOOKUP_ATTRIBUTE" goto trim_spaces
if "%VAR_NAME%" == "SEARCH_PW" goto trim_spaces
if "%VAR_NAME%" == "USER_BASEDN" goto trim_spaces
if "%VAR_NAME%" == "GROUP_BASEDN" goto trim_spaces
if "%VAR_NAME%" == "SEARCH_DN" goto trim_spaces
exit /b
:trim_spaces
for /l %%a in (1,1,1000) do (
    if "!VAR_VALUE:~-1!" == " " (
       set !VAR_VALUE!=!VAR_VALUE:~0,-1!
    ) else (
       exit /b
    )
)
exit /b

endlocal